@@ -161,6 +161,9 @@ public:
 
     QList<QPointer<Perspective>> m_perspectives;
     QSet<QString> m_persistentChangedDocks;
+
+    QHash<QString, QByteArray> m_lastPerspectiveStates;
+    QHash<QString, QByteArray> m_lastTypePerspectiveStates;
 };
 
 DebuggerMainWindowPrivate::DebuggerMainWindowPrivate(DebuggerMainWindow *parent)
@@ -169,14 +172,14 @@ DebuggerMainWindowPrivate::DebuggerMainWindowPrivate(DebuggerMainWindow *parent)
     m_centralWidgetStack = new QStackedWidget;
     m_statusLabel = new Utils::StatusLabel;
     m_statusLabel->setProperty("panelwidget", true);
-    m_statusLabel->setIndent(2 * QFontMetrics(q->font()).width(QChar('x')));
+    m_statusLabel->setIndent(2 * QFontMetrics(q->font()).horizontalAdvance(QChar('x')));
     m_editorPlaceHolder = new EditorManagerPlaceHolder;
 
     m_perspectiveChooser = new QComboBox;
     m_perspectiveChooser->setObjectName("PerspectiveChooser");
     m_perspectiveChooser->setProperty("panelwidget", true);
     m_perspectiveChooser->setSizeAdjustPolicy(QComboBox::AdjustToContents);
-    connect(m_perspectiveChooser, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
+    connect(m_perspectiveChooser, QOverload<int>::of(&QComboBox::activated),
             this, [this](int item) {
         Perspective *perspective = Perspective::findPerspective(m_perspectiveChooser->itemData(item).toString());
         QTC_ASSERT(perspective, return);
@@ -285,13 +288,12 @@ DebuggerMainWindow::DebuggerMainWindow()
     cmd->setAttribute(Command::CA_Hide);
     viewsMenu->addAction(cmd, Core::Constants::G_DEFAULT_THREE);
 
-    connect(ICore::instance(), &ICore::saveSettingsRequested, this, [this] {
+    connect(ICore::instance(), &ICore::saveSettingsRequested, this,
+            [this](ICore::SaveSettingsReason reason) {
         // There's one saveSettings triggered after plugin loading intentionally.
         // We do not want to save anything at that time.
-        static bool firstOne = true;
-        if (firstOne) {
+        if (reason == ICore::InitializationDone) {
             qCDebug(perspectivesLog) << "FIRST SAVE SETTINGS REQUEST IGNORED";
-            firstOne = false;
         } else {
             qCDebug(perspectivesLog) << "SAVING SETTINGS";
             savePersistentSettings();
@@ -431,15 +433,16 @@ void DebuggerMainWindow::restorePersistentSettings()
     qCDebug(perspectivesLog) << "RESTORE PERSISTENT";
     QSettings *settings = ICore::settings();
     settings->beginGroup(MAINWINDOW_KEY);
-    const bool res = theMainWindow->restoreState(settings->value(STATE_KEY).toByteArray(),
-                                                 SettingsVersion);
-    if (!res)
-        qCDebug(perspectivesLog) << "NO READABLE PERSISTENT SETTINGS FOUND, ASSUMING NEW CLEAN SETTINGS";
+
+    const QHash<QString, QVariant> states = settings->value(STATE_KEY).toHash();
+    theMainWindow->d->m_lastTypePerspectiveStates.clear();
+    for (const QString &type : states.keys())
+        theMainWindow->d->m_lastTypePerspectiveStates.insert(type, states.value(type).toByteArray());
 
     theMainWindow->setAutoHideTitleBars(settings->value(AUTOHIDE_TITLEBARS_KEY, true).toBool());
     theMainWindow->showCentralWidget(settings->value(SHOW_CENTRALWIDGET_KEY, true).toBool());
     theMainWindow->d->m_persistentChangedDocks
-            = QSet<QString>::fromList(settings->value(CHANGED_DOCK_KEY).toStringList());
+            = Utils::toSet(settings->value(CHANGED_DOCK_KEY).toStringList());
     settings->endGroup();
 
     qCDebug(perspectivesLog) << "LOADED DOCKS:" << theMainWindow->d->m_persistentChangedDocks;
@@ -497,10 +500,14 @@ void DebuggerMainWindow::savePersistentSettings()
     theMainWindow->d->m_persistentChangedDocks = changedDocks;
     qCDebug(perspectivesLog) << "CHANGED DOCKS:" << changedDocks;
 
+    QVariantHash states;
+    for (const QString &type : theMainWindow->d->m_lastTypePerspectiveStates.keys())
+        states.insert(type, QVariant::fromValue(theMainWindow->d->m_lastTypePerspectiveStates.value(type)));
+
     QSettings *settings = ICore::settings();
     settings->beginGroup(MAINWINDOW_KEY);
-    settings->setValue(CHANGED_DOCK_KEY, QStringList(changedDocks.toList()));
-    settings->setValue(STATE_KEY, theMainWindow->saveState(SettingsVersion));
+    settings->setValue(CHANGED_DOCK_KEY, QStringList(Utils::toList(changedDocks)));
+    settings->setValue(STATE_KEY, states);
     settings->setValue(AUTOHIDE_TITLEBARS_KEY, theMainWindow->autoHideTitleBars());
     settings->setValue(SHOW_CENTRALWIDGET_KEY, theMainWindow->isCentralWidgetShown());
     settings->endGroup();
@@ -634,7 +641,8 @@ void DebuggerMainWindowPrivate::updatePerspectiveChooserWidth()
     if (index != -1) {
         m_perspectiveChooser->setCurrentIndex(index);
 
-        const int contentWidth = m_perspectiveChooser->fontMetrics().width(perspective->d->m_name);
+        const int contentWidth =
+            m_perspectiveChooser->fontMetrics().horizontalAdvance(perspective->d->m_name);
         QStyleOptionComboBox option;
         option.initFrom(m_perspectiveChooser);
         const QSize sz(contentWidth, 1);
@@ -940,6 +948,13 @@ void PerspectivePrivate::restoreLayout()
                                      << (active == op.visibleByDefault ? "DEFAULT USER" : "*** NON-DEFAULT USER");
         }
     }
+
+    QByteArray state;
+    if (theMainWindow->d->m_lastTypePerspectiveStates.contains(settingsId()))
+        state = theMainWindow->d->m_lastTypePerspectiveStates.value(settingsId());
+    if (theMainWindow->d->m_lastPerspectiveStates.contains(m_id))
+        state = theMainWindow->d->m_lastPerspectiveStates.value(m_id);
+    theMainWindow->restoreState(state);
 }
 
 void PerspectivePrivate::saveLayout()
@@ -959,6 +974,8 @@ void PerspectivePrivate::saveLayout()
                                      << (active == op.visibleByDefault ? "DEFAULT USER" : "*** NON-DEFAULT USER");
         }
     }
+    theMainWindow->d->m_lastPerspectiveStates.insert(m_id, theMainWindow->saveState());
+    theMainWindow->d->m_lastTypePerspectiveStates.insert(settingsId(), theMainWindow->saveState());
 }
 
 QString PerspectivePrivate::settingsId() const
