@@ -27,6 +27,7 @@
 
 #include "abi.h"
 #include "kitinformation.h"
+#include "msvctoolchain.h"
 #include "toolchain.h"
 #include "toolchainsettingsaccessor.h"
 
@@ -62,11 +63,12 @@ class ToolChainManagerPrivate
 public:
     ~ToolChainManagerPrivate();
 
-    QMap<QString, FileName> m_abiToDebugger;
+    QMap<QString, FilePath> m_abiToDebugger;
     std::unique_ptr<ToolChainSettingsAccessor> m_accessor;
 
     QList<ToolChain *> m_toolChains; // prioritized List
     QVector<LanguageDisplayPair> m_languages;
+    ToolchainDetectionSettings m_detectionSettings;
 };
 
 ToolChainManagerPrivate::~ToolChainManagerPrivate()
@@ -76,12 +78,14 @@ ToolChainManagerPrivate::~ToolChainManagerPrivate()
 }
 
 static ToolChainManager *m_instance = nullptr;
-static ToolChainManagerPrivate *d;
+static ToolChainManagerPrivate *d = nullptr;
 
 } // namespace Internal
 
 using namespace Internal;
 
+const char DETECT_X64_AS_X32_KEY[] = "ProjectExplorer/Toolchains/DetectX64AsX32";
+
 // --------------------------------------------------------------------------
 // ToolChainManager
 // --------------------------------------------------------------------------
@@ -99,12 +103,16 @@ ToolChainManager::ToolChainManager(QObject *parent) :
     connect(this, &ToolChainManager::toolChainAdded, this, &ToolChainManager::toolChainsChanged);
     connect(this, &ToolChainManager::toolChainRemoved, this, &ToolChainManager::toolChainsChanged);
     connect(this, &ToolChainManager::toolChainUpdated, this, &ToolChainManager::toolChainsChanged);
+
+    QSettings * const s = Core::ICore::settings();
+    d->m_detectionSettings.detectX64AsX32 = s->value(DETECT_X64_AS_X32_KEY, false).toBool();
 }
 
 ToolChainManager::~ToolChainManager()
 {
-    delete d;
     m_instance = nullptr;
+    delete d;
+    d = nullptr;
 }
 
 ToolChainManager *ToolChainManager::instance()
@@ -128,6 +136,8 @@ void ToolChainManager::saveToolChains()
     QTC_ASSERT(d->m_accessor, return);
 
     d->m_accessor->saveToolChains(d->m_toolChains, Core::ICore::dialogParent());
+    QSettings * const s = Core::ICore::settings();
+    s->setValue(DETECT_X64_AS_X32_KEY, d->m_detectionSettings.detectX64AsX32);
 }
 
 QList<ToolChain *> ToolChainManager::toolChains(const ToolChain::Predicate &predicate)
@@ -176,7 +186,7 @@ ToolChain *ToolChainManager::findToolChain(const QByteArray &id)
     return tc;
 }
 
-FileName ToolChainManager::defaultDebugger(const Abi &abi)
+FilePath ToolChainManager::defaultDebugger(const Abi &abi)
 {
     return d->m_abiToDebugger.value(abi.toString());
 }
@@ -248,4 +258,21 @@ bool ToolChainManager::isLanguageSupported(const Core::Id &id)
     return Utils::contains(d->m_languages, Utils::equal(&LanguageDisplayPair::id, id));
 }
 
+void ToolChainManager::aboutToShutdown()
+{
+#ifdef Q_OS_WIN
+    MsvcToolChain::cancelMsvcToolChainDetection();
+#endif
+}
+
+ToolchainDetectionSettings ToolChainManager::detectionSettings()
+{
+    return d->m_detectionSettings;
+}
+
+void ToolChainManager::setDetectionSettings(const ToolchainDetectionSettings &settings)
+{
+    d->m_detectionSettings = settings;
+}
+
 } // namespace ProjectExplorer
