@@ -63,30 +63,26 @@ using namespace Internal;
 const char QTVERSION_DATA_KEY[] = "QtVersion.";
 const char QTVERSION_TYPE_KEY[] = "QtVersion.Type";
 const char QTVERSION_FILE_VERSION_KEY[] = "Version";
-const char QTVERSION_FILENAME[] = "/qtcreator/qtversion.xml";
+const char QTVERSION_FILENAME[] = "/qtversion.xml";
 
 static QMap<int, BaseQtVersion *> m_versions;
 static int m_idcount = 0;
 // managed by QtProjectManagerPlugin
-static QtVersionManager *m_instance = 0;
-static FileSystemWatcher *m_configFileWatcher = 0;
-static QTimer *m_fileWatcherTimer = 0;
-static PersistentSettingsWriter *m_writer = 0;
+static QtVersionManager *m_instance = nullptr;
+static FileSystemWatcher *m_configFileWatcher = nullptr;
+static QTimer *m_fileWatcherTimer = nullptr;
+static PersistentSettingsWriter *m_writer = nullptr;
 
 enum { debug = 0 };
 
-static FileName globalSettingsFileName()
+static FilePath globalSettingsFileName()
 {
-    QSettings *globalSettings = ExtensionSystem::PluginManager::globalSettings();
-    return FileName::fromString(QFileInfo(globalSettings->fileName()).absolutePath()
-                                       + QLatin1String(QTVERSION_FILENAME));
+    return FilePath::fromString(Core::ICore::installerResourcePath() + QTVERSION_FILENAME);
 }
 
-static FileName settingsFileName(const QString &path)
+static FilePath settingsFileName(const QString &path)
 {
-    QSettings *settings = ExtensionSystem::PluginManager::settings();
-    QFileInfo settingsLocation(settings->fileName());
-    return FileName::fromString(settingsLocation.absolutePath() + path);
+    return FilePath::fromString(Core::ICore::userResourcePath() + path);
 }
 
 
@@ -108,12 +104,12 @@ static void updateDocumentation();
 QtVersionManager::QtVersionManager()
 {
     m_instance = this;
-    m_configFileWatcher = 0;
+    m_configFileWatcher = nullptr;
     m_fileWatcherTimer = new QTimer(this);
-    m_writer = 0;
+    m_writer = nullptr;
     m_idcount = 1;
 
-    qRegisterMetaType<FileName>();
+    qRegisterMetaType<FilePath>();
 
     // Give the file a bit of time to settle before reading it...
     m_fileWatcherTimer->setInterval(2000);
@@ -138,11 +134,11 @@ void QtVersionManager::triggerQtVersionRestore()
     emit m_instance->qtVersionsChanged(m_versions.keys(), QList<int>(), QList<int>());
     saveQtVersions();
 
-    const FileName configFileName = globalSettingsFileName();
+    const FilePath configFileName = globalSettingsFileName();
     if (configFileName.exists()) {
         m_configFileWatcher = new FileSystemWatcher(m_instance);
         connect(m_configFileWatcher, &FileSystemWatcher::fileChanged,
-                m_fileWatcherTimer, static_cast<void (QTimer::*)()>(&QTimer::start));
+                m_fileWatcherTimer, QOverload<>::of(&QTimer::start));
         m_configFileWatcher->addFile(configFileName.toString(),
                                      FileSystemWatcher::WatchModifiedDate);
     } // exists
@@ -179,10 +175,10 @@ static bool restoreQtVersions()
     m_writer = new PersistentSettingsWriter(settingsFileName(QLatin1String(QTVERSION_FILENAME)),
                                                    QLatin1String("QtCreatorQtVersions"));
 
-    QList<QtVersionFactory *> factories = ExtensionSystem::PluginManager::getObjects<QtVersionFactory>();
+    const QList<QtVersionFactory *> factories = QtVersionFactory::allQtVersionFactories();
 
     PersistentSettingsReader reader;
-    FileName filename = settingsFileName(QLatin1String(QTVERSION_FILENAME));
+    FilePath filename = settingsFileName(QLatin1String(QTVERSION_FILENAME));
 
     if (!reader.load(filename))
         return false;
@@ -208,7 +204,7 @@ static bool restoreQtVersions()
         const QString type = qtversionMap.value(QLatin1String(QTVERSION_TYPE_KEY)).toString();
 
         bool restored = false;
-        foreach (QtVersionFactory *f, factories) {
+        for (QtVersionFactory *f : factories) {
             if (f->canRestore(type)) {
                 if (BaseQtVersion *qtv = f->restore(type, qtversionMap)) {
                     if (m_versions.contains(qtv->uniqueId())) {
@@ -238,7 +234,7 @@ void QtVersionManager::updateFromInstaller(bool emitSignal)
 {
     m_fileWatcherTimer->stop();
 
-    const FileName path = globalSettingsFileName();
+    const FilePath path = globalSettingsFileName();
     // Handle overwritting of data:
     if (m_configFileWatcher) {
         m_configFileWatcher->removeFile(path.toString());
@@ -249,7 +245,7 @@ void QtVersionManager::updateFromInstaller(bool emitSignal)
     QList<int> removed;
     QList<int> changed;
 
-    QList<QtVersionFactory *> factories = ExtensionSystem::PluginManager::getObjects<QtVersionFactory>();
+    const QList<QtVersionFactory *> factories = QtVersionFactory::allQtVersionFactories();
     PersistentSettingsReader reader;
     QVariantMap data;
     if (reader.load(path))
@@ -283,7 +279,7 @@ void QtVersionManager::updateFromInstaller(bool emitSignal)
         const QString autoDetectionSource = qtversionMap.value(QLatin1String("autodetectionSource")).toString();
         sdkVersions << autoDetectionSource;
         int id = -1; // see BaseQtVersion::fromMap()
-        QtVersionFactory *factory = 0;
+        QtVersionFactory *factory = nullptr;
         foreach (QtVersionFactory *f, factories) {
             if (f->canRestore(type))
                 factory = f;
@@ -412,33 +408,31 @@ static QString qmakePath(const QString &qtchooser, const QString &version)
     return QString();
 }
 
-static FileNameList gatherQmakePathsFromQtChooser()
+static FilePathList gatherQmakePathsFromQtChooser()
 {
     const QString qtchooser = QStandardPaths::findExecutable(QStringLiteral("qtchooser"));
     if (qtchooser.isEmpty())
-        return FileNameList();
+        return FilePathList();
 
-    QList<QByteArray> versions = runQtChooser(qtchooser, QStringList() << QStringLiteral("-l"));
-    QSet<FileName> foundQMakes;
+    QList<QByteArray> versions = runQtChooser(qtchooser, QStringList("-l"));
+    QSet<FilePath> foundQMakes;
     foreach (const QByteArray &version, versions) {
-        FileName possibleQMake = FileName::fromString(
+        FilePath possibleQMake = FilePath::fromString(
                     qmakePath(qtchooser, QString::fromLocal8Bit(version)));
         if (!possibleQMake.isEmpty())
             foundQMakes << possibleQMake;
     }
-    return foundQMakes.toList();
+    return Utils::toList(foundQMakes);
 }
 
 static void findSystemQt()
 {
-    FileNameList systemQMakes;
-    FileName systemQMakePath = BuildableHelperLibrary::findSystemQt(Environment::systemEnvironment());
-    if (!systemQMakePath.isEmpty())
-        systemQMakes << systemQMakePath;
+    FilePathList systemQMakes
+            = BuildableHelperLibrary::findQtsInEnvironment(Environment::systemEnvironment());
 
     systemQMakes.append(gatherQmakePathsFromQtChooser());
 
-    foreach (const FileName &qmakePath, Utils::filteredUnique(systemQMakes)) {
+    foreach (const FilePath &qmakePath, Utils::filteredUnique(systemQMakes)) {
         BaseQtVersion *version
                 = QtVersionFactory::createQtVersionFromQMakePath(qmakePath, false, QLatin1String("PATH"));
         if (version) {
@@ -451,7 +445,7 @@ static void findSystemQt()
 void QtVersionManager::addVersion(BaseQtVersion *version)
 {
     QTC_ASSERT(m_writer, return);
-    QTC_ASSERT(version != 0, return);
+    QTC_ASSERT(version, return);
     if (m_versions.contains(version->uniqueId()))
         return;
 
@@ -464,7 +458,7 @@ void QtVersionManager::addVersion(BaseQtVersion *version)
 
 void QtVersionManager::removeVersion(BaseQtVersion *version)
 {
-    QTC_ASSERT(version != 0, return);
+    QTC_ASSERT(version, return);
     m_versions.remove(version->uniqueId());
     emit m_instance->qtVersionsChanged(QList<int>(), QList<int>() << version->uniqueId(), QList<int>());
     saveQtVersions();
@@ -475,59 +469,38 @@ static void updateDocumentation()
 {
     QStringList files;
     foreach (BaseQtVersion *v, m_versions) {
-        const QStringList docPaths = QStringList() << v->documentationPath() + QLatin1Char('/')
-                                                   << v->documentationPath() + QLatin1String("/qch/");
+        const QStringList docPaths = QStringList({v->documentationPath() + QChar('/'),
+                                                  v->documentationPath() + "/qch/"});
         foreach (const QString &docPath, docPaths) {
             const QDir versionHelpDir(docPath);
             foreach (const QString &helpFile,
-                     versionHelpDir.entryList(QStringList() << QLatin1String("*.qch"), QDir::Files))
+                     versionHelpDir.entryList(QStringList("*.qch"), QDir::Files))
                 files << docPath + helpFile;
         }
     }
     Core::HelpManager::registerDocumentation(files);
 }
 
-void QtVersionManager::updateDumpFor(const FileName &qmakeCommand)
-{
-    foreach (BaseQtVersion *v, unsortedVersions()) {
-        if (v->qmakeCommand() == qmakeCommand)
-            v->recheckDumper();
-    }
-    emit dumpUpdatedFor(qmakeCommand);
-}
-
 int QtVersionManager::getUniqueId()
 {
     return m_idcount++;
 }
 
-QList<BaseQtVersion *> QtVersionManager::unsortedVersions()
+QList<BaseQtVersion *> QtVersionManager::versions(const BaseQtVersion::Predicate &predicate)
 {
     QList<BaseQtVersion *> versions;
     QTC_ASSERT(isLoaded(), return versions);
-    return m_versions.values();
+    if (predicate)
+        return Utils::filtered(m_versions.values(), predicate);
+    else
+        return m_versions.values();
 }
 
-QList<BaseQtVersion *> QtVersionManager::versions()
+QList<BaseQtVersion *> QtVersionManager::sortVersions(const QList<BaseQtVersion *> &input)
 {
-    QList<BaseQtVersion *> versions;
-    QTC_ASSERT(isLoaded(), return versions);
-    foreach (BaseQtVersion *version, m_versions)
-        versions << version;
-    Utils::sort(versions, qtVersionNumberCompare);
-    return versions;
-}
-
-QList<BaseQtVersion *> QtVersionManager::validVersions()
-{
-    QList<BaseQtVersion *> results;
-    QTC_ASSERT(isLoaded(), return results);
-    foreach (BaseQtVersion *v, m_versions) {
-        if (v->isValid())
-            results.append(v);
-    }
-    Utils::sort(results, qtVersionNumberCompare);
-    return results;
+    QList<BaseQtVersion *> result = input;
+    Utils::sort(result, qtVersionNumberCompare);
+    return result;
 }
 
 bool QtVersionManager::isValidId(int id)
@@ -538,13 +511,18 @@ bool QtVersionManager::isValidId(int id)
 
 BaseQtVersion *QtVersionManager::version(int id)
 {
-    QTC_ASSERT(isLoaded(), return 0);
+    QTC_ASSERT(isLoaded(), return nullptr);
     QMap<int, BaseQtVersion *>::const_iterator it = m_versions.constFind(id);
     if (it == m_versions.constEnd())
-        return 0;
+        return nullptr;
     return it.value();
 }
 
+BaseQtVersion *QtVersionManager::version(const BaseQtVersion::Predicate &predicate)
+{
+    return Utils::findOrDefault(m_versions.values(), predicate);
+}
+
 // This function is really simplistic...
 static bool equals(BaseQtVersion *a, BaseQtVersion *b)
 {
@@ -556,9 +534,7 @@ void QtVersionManager::setNewQtVersions(QList<BaseQtVersion *> newVersions)
     // We want to preserve the same order as in the settings dialog
     // so we sort a copy
     QList<BaseQtVersion *> sortedNewVersions = newVersions;
-    Utils::sort(sortedNewVersions, [](const BaseQtVersion *l, const BaseQtVersion *r) {
-        return l->uniqueId() < r->uniqueId();
-    });
+    Utils::sort(sortedNewVersions, &BaseQtVersion::uniqueId);
 
     QList<int> addedVersions;
     QList<int> removedVersions;
@@ -615,9 +591,9 @@ void QtVersionManager::setNewQtVersions(QList<BaseQtVersion *> newVersions)
         emit m_instance->qtVersionsChanged(addedVersions, removedVersions, changedVersions);
 }
 
-BaseQtVersion *QtVersionManager::qtVersionForQMakeBinary(const FileName &qmakePath)
+BaseQtVersion *QtVersionManager::qtVersionForQMakeBinary(const FilePath &qmakePath)
 {
-    return Utils::findOrDefault(unsortedVersions(), Utils::equal(&BaseQtVersion::qmakeCommand, qmakePath));
+    return version(Utils::equal(&BaseQtVersion::qmakeCommand, qmakePath));
 }
 
 } // namespace QtVersion
