@@ -30,7 +30,7 @@
 #include <texteditor/completionsettings.h>
 
 #include <QDebug>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QtAlgorithms>
 #include <QHash>
 
@@ -38,15 +38,17 @@
 
 using namespace TextEditor;
 
+QT_BEGIN_NAMESPACE
 uint qHash(const AssistProposalItem &item)
 {
     return qHash(item.text());
 }
+QT_END_NAMESPACE
 
 namespace {
 
-const int kMaxSort = 1000;
-const int kMaxPrefixFilter = 100;
+constexpr int kMaxSort = 1000;
+constexpr int kMaxPrefixFilter = 100;
 
 struct ContentLessThan
 {
@@ -129,8 +131,7 @@ private:
 
 } // Anonymous
 
-GenericProposalModel::GenericProposalModel()
-{}
+GenericProposalModel::GenericProposalModel() = default;
 
 GenericProposalModel::~GenericProposalModel()
 {
@@ -141,12 +142,103 @@ void GenericProposalModel::loadContent(const QList<AssistProposalItemInterface *
 {
     m_originalItems = items;
     m_currentItems = items;
+    m_duplicatesRemoved = false;
     for (int i = 0; i < m_originalItems.size(); ++i)
         m_idByText.insert(m_originalItems.at(i)->text(), i);
 }
 
+bool GenericProposalModel::hasItemsToPropose(const QString &prefix, AssistReason reason) const
+{
+    return size() != 0 && (keepPerfectMatch(reason) || !isPerfectMatch(prefix));
+}
+
+static QString cleanText(const QString &original)
+{
+    QString clean = original;
+    int ignore = 0;
+    for (int i = clean.length() - 1; i >= 0; --i, ++ignore) {
+        const QChar &c = clean.at(i);
+        if (c.isLetterOrNumber() || c == QLatin1Char('_')
+                || c.isHighSurrogate() || c.isLowSurrogate()) {
+            break;
+        }
+    }
+    if (ignore)
+        clean.chop(ignore);
+    return clean;
+}
+
+static bool textStartsWith(CaseSensitivity cs, const QString &text, const QString &prefix)
+{
+    switch (cs) {
+    case TextEditor::CaseInsensitive:
+        return text.startsWith(prefix, Qt::CaseInsensitive);
+    case TextEditor::CaseSensitive:
+        return text.startsWith(prefix, Qt::CaseSensitive);
+    case TextEditor::FirstLetterCaseSensitive:
+        return prefix.at(0) == text.at(0)
+               && prefix.midRef(1).startsWith(text.midRef(1), Qt::CaseInsensitive);
+    }
+
+    return false;
+}
+
+enum class PerfectMatchType {
+    None,
+    StartsWith,
+    Full,
+};
+
+static PerfectMatchType perfectMatch(CaseSensitivity cs, const QString &text, const QString &prefix)
+{
+    if (textStartsWith(cs, text, prefix))
+        return prefix.size() == text.size() ? PerfectMatchType::Full : PerfectMatchType::StartsWith;
+
+    return PerfectMatchType::None;
+}
+
+bool GenericProposalModel::isPerfectMatch(const QString &prefix) const
+{
+    if (prefix.isEmpty())
+        return false;
+
+    const CaseSensitivity cs = TextEditorSettings::completionSettings().m_caseSensitivity;
+    bool hasFullMatch = false;
+
+    for (int i = 0; i < size(); ++i) {
+        const QString &current = cleanText(text(i));
+        if (current.isEmpty())
+            continue;
+
+        const PerfectMatchType match = perfectMatch(cs, current, prefix);
+        if (match == PerfectMatchType::StartsWith)
+            return false;
+
+        if (match == PerfectMatchType::Full) {
+            if (proposalItem(i)->isKeyword())
+                return true;
+
+            if (!hasFullMatch)
+                hasFullMatch = true;
+        }
+    }
+
+    return hasFullMatch;
+}
+
+bool GenericProposalModel::isPrefiltered(const QString &prefix) const
+{
+    return !m_prefilterPrefix.isEmpty() && prefix == m_prefilterPrefix;
+}
+
+void GenericProposalModel::setPrefilterPrefix(const QString &prefix)
+{
+    m_prefilterPrefix = prefix;
+}
+
 void GenericProposalModel::reset()
 {
+    m_prefilterPrefix.clear();
     m_currentItems = m_originalItems;
 }
 
@@ -170,8 +262,16 @@ QString GenericProposalModel::detail(int index) const
     return m_currentItems.at(index)->detail();
 }
 
+Qt::TextFormat GenericProposalModel::detailFormat(int index) const
+{
+    return m_currentItems.at(index)->detailFormat();
+}
+
 void GenericProposalModel::removeDuplicates()
 {
+    if (m_duplicatesRemoved)
+        return;
+
     QHash<QString, quint64> unique;
     auto it = m_originalItems.begin();
     while (it != m_originalItems.end()) {
@@ -185,6 +285,8 @@ void GenericProposalModel::removeDuplicates()
             ++it;
         }
     }
+
+    m_duplicatesRemoved = true;
 }
 
 void GenericProposalModel::filter(const QString &prefix)
@@ -192,63 +294,40 @@ void GenericProposalModel::filter(const QString &prefix)
     if (prefix.isEmpty())
         return;
 
-    /*
-     * This code builds a regular expression in order to more intelligently match
-     * camel-case and underscore names.
-     *
-     * For any but the first letter, the following replacements are made:
-     *   A => [a-z0-9_]*A
-     *   a => (?:[a-zA-Z0-9]*_)?a
-     *
-     * That means any sequence of lower-case or underscore characters can preceed an
-     * upper-case character. And any sequence of lower-case or upper case characters -
-     * followed by an underscore can preceed a lower-case character.
-     *
-     * Examples: (case sensitive mode)
-     *   gAC matches getActionController
-     *   gac matches get_action_controller
-     *
-     * It also implements the fully and first-letter-only case sensitivity.
-     */
-    const CaseSensitivity caseSensitivity =
-        TextEditorSettings::completionSettings().m_caseSensitivity;
-
-    QString keyRegExp;
-    keyRegExp += QLatin1Char('^');
-    bool first = true;
-    const QLatin1String uppercaseWordContinuation("[a-z0-9_]*");
-    const QLatin1String lowercaseWordContinuation("(?:[a-zA-Z0-9]*_)?");
-    foreach (const QChar &c, prefix) {
-        if (caseSensitivity == CaseInsensitive ||
-            (caseSensitivity == FirstLetterCaseSensitive && !first)) {
-
-            keyRegExp += QLatin1String("(?:");
-            if (!first)
-                keyRegExp += uppercaseWordContinuation;
-            keyRegExp += QRegExp::escape(c.toUpper());
-            keyRegExp += QLatin1Char('|');
-            if (!first)
-                keyRegExp += lowercaseWordContinuation;
-            keyRegExp += QRegExp::escape(c.toLower());
-            keyRegExp += QLatin1Char(')');
-        } else {
-            if (!first) {
-                if (c.isUpper())
-                    keyRegExp += uppercaseWordContinuation;
-                else
-                    keyRegExp += lowercaseWordContinuation;
-            }
-            keyRegExp += QRegExp::escape(c);
-        }
-
-        first = false;
-    }
-    QRegExp regExp(keyRegExp);
+    const FuzzyMatcher::CaseSensitivity caseSensitivity =
+        convertCaseSensitivity(TextEditorSettings::completionSettings().m_caseSensitivity);
+    const QRegularExpression regExp = FuzzyMatcher::createRegExp(prefix, caseSensitivity);
 
     m_currentItems.clear();
+    const QString lowerPrefix = prefix.toLower();
     foreach (const auto &item, m_originalItems) {
-        if (regExp.indexIn(item->text()) == 0)
+        const QString &text = item->text();
+        if (regExp.match(text).capturedStart() == 0) {
             m_currentItems.append(item);
+            if (text.startsWith(prefix)) {
+                // Direct match
+                item->setPrefixMatch(text.length() == prefix.length()
+                                     ? AssistProposalItemInterface::PrefixMatch::Full
+                                     : AssistProposalItemInterface::PrefixMatch::Exact);
+                continue;
+            }
+
+            if (text.startsWith(lowerPrefix, Qt::CaseInsensitive))
+                item->setPrefixMatch(AssistProposalItemInterface::PrefixMatch::Lower);
+        }
+    }
+}
+
+FuzzyMatcher::CaseSensitivity
+    GenericProposalModel::convertCaseSensitivity(TextEditor::CaseSensitivity textEditorCaseSensitivity)
+{
+    switch (textEditorCaseSensitivity) {
+    case TextEditor::CaseSensitive:
+        return FuzzyMatcher::CaseSensitivity::CaseSensitive;
+    case TextEditor::FirstLetterCaseSensitive:
+        return FuzzyMatcher::CaseSensitivity::FirstLetterCaseSensitive;
+    default:
+        return FuzzyMatcher::CaseSensitivity::CaseInsensitive;
     }
 }
 
